#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'FirePass (F5 Networks)'


def is_waf(self):
    schema1 = [
        self.matchCookie('^VHOST'),
        self.matchHeader(('Location', r'\/my\.logon\.php3'))
    ]
    schema2 = [
        self.matchCookie(r'^F5_fire.+?'),
        self.matchCookie('^F5_passid_shrinked')
    ]
    if all(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Chuang Yu Shield (Yunaq)'


def is_waf(self):
    schemes = [
        self.matchContent(r'www\.365cyd\.com'),
        self.matchContent(r'help\.365cyd\.com/cyd\-error\-help.html\?code=403')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'PerimeterX (PerimeterX)'


def is_waf(self):
    schemes = [
        self.matchContent(r'www\.perimeterx\.(com|net)/whywasiblocked'),
        self.matchContent(r'client\.perimeterx\.(net|com)'),
        self.matchContent(r'denied because we believe you are using automation tools')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DotDefender (Applicure Technologies)'


def is_waf(self):
    schemes = [
        self.matchHeader(('X-dotDefender-denied', r'.+?'), attack=True),
        self.matchContent(r'dotdefender blocked your request'),
        self.matchContent(r'Applicure is the leading provider of web application security')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Bekchy (Faydata Technologies Inc.)'


def is_waf(self):
    schemes = [
        # Both signatures are contained within response, so checking for any one of them
        # Sometimes I observed that there is an XHR request being being made to submit the 
        # report data automatically upon page load. In those cases a missing https is causing
        # false negatives.
        self.matchContent(r'Bekchy.{0,10}?Access Denied'),
        self.matchContent(r'bekchy\.com/report')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'AliYunDun (Alibaba Cloud Computing)'


def is_waf(self):
    schemes = [
        self.matchContent(r'error(s)?\.aliyun(dun)?\.(com|net)?'),
        self.matchCookie(r'^aliyungf_tc='),
        self.matchContent(r'cdn\.aliyun(cs)?\.com'),
        self.matchStatus(405)
        ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Safe3 Web Firewall (Safe3)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', 'Safe3 Web Firewall')),
        self.matchHeader(('X-Powered-By', r'Safe3WAF/[\.0-9]+?')),
        self.matchContent(r'Safe3waf/[0-9\.]+?')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

import re
import sys
import time
import logging
from copy import copy

import requests
import urllib3
try:
    from urlparse import urlparse, urlunparse
except ImportError:
    from urllib.parse import urlparse, urlunparse
try:
    from urllib import quote, unquote
except ImportError:
    from urllib.parse import quote, unquote
# For requests < 2.16, this should be used.
# requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
# For requests >= 2.16, this is the convention 
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def_headers = {'Accept'         : 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
               'Accept-Encoding': 'gzip, deflate',
               'Accept-Language': 'en-US,en;q=0.9',
               'DNT'            : '1',  # Do Not Track request header
               'User-Agent'     : 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3770.100 Safari/537.36',
               'Upgrade-Insecure-Requests': '1' #
        }
proxies = {}

def urlParser(target):
    log = logging.getLogger('urlparser')

    ssl = False
    o = urlparse(target)
    if o[0] not in ['http', 'https', '']:
        log.error('scheme %s not supported' % o[0])
        return
    if o[0] == 'https':
        ssl = True
    if len(o[2]) > 0:
        path = o[2]
    else:
        path = '/'
    tmp = o[1].split(':')
    if len(tmp) > 1:
        port = tmp[1]
    else:
        port = None
    hostname = tmp[0]
    query = o[4]
    return (hostname, port, path, query, ssl)

class waftoolsengine:
    def __init__(self, target='https://example.com', debuglevel=0, path='/', proxies=None, 
                redir=True, head=None):
        self.target = target
        self.debuglevel = debuglevel
        self.requestnumber = 0
        self.path = path
        self.redirectno = 0
        self.allowredir = redir
        self.proxies = proxies
        self.log = logging.getLogger('wafw00f')
        if head:
            self.headers = head
        else:
            self.headers = copy(def_headers) #copy object by value not reference. Fix issue #90

    def Request(self, headers=None, path=None, params={}, delay=0, timeout=7):
        try:
            time.sleep(delay)
            if not headers: 
                h = self.headers
            else: h = headers
            req = requests.get(self.target, proxies=self.proxies, headers=h, timeout=timeout,
                    allow_redirects=self.allowredir, params=params, verify=False)
            self.log.info('Request Succeeded')
            self.log.debug('Headers: %s\n' % req.headers)
            self.log.debug('Content: %s\n' % req.content)
            self.requestnumber += 1
            return req
        except requests.exceptions.RequestException as e:
            self.log.error('Something went wrong %s' % (e.__str__()))



#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

import os
from functools import partial
from pluginbase import PluginBase

def load_plugins():
    here = os.path.abspath(os.path.dirname(__file__))
    get_path = partial(os.path.join, here)
    plugin_dir = get_path('plugins')

    plugin_base = PluginBase(
        package='wafw00f.plugins', searchpath=[plugin_dir]
    )
    plugin_source = plugin_base.make_plugin_source(
        searchpath=[plugin_dir], persist=True
    )

    plugin_dict = {}
    for plugin_name in plugin_source.list_plugins():
        plugin_dict[plugin_name] = plugin_source.load_plugin(plugin_name)

    return plugin_dict



#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'AppWall (Radware)'


def is_waf(self):
    schema1 = [
        self.matchContent(r'CloudWebSec\.radware\.com'),
        self.matchHeader(('X-SL-CompState', '.+'))
    ]
    schema2 = [
        self.matchContent(r'because we have detected unauthorized activity'),
        self.matchContent(r'<title>Unauthorized Request Blocked'),
        self.matchContent(r'if you believe that there has been some mistake'),
        self.matchContent(r'\?Subject=Security Page.{0,10}?Case Number')
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ZScaler (Accenture)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'ZScaler')),
        self.matchContent(r"Access Denied.{0,10}?Accenture Policy"),
        self.matchContent(r'policies\.accenture\.com'),
        self.matchContent(r'login\.zscloud\.net/img_logo_new1\.png'),
        self.matchContent(r'Zscaler to protect you from internet threats'),
        self.matchContent(r"Internet Security by ZScaler"),
        self.matchContent(r"Accenture.{0,10}?webfilters indicate that the site likely contains")
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'OnMessage Shield (BlackBaud)'


def is_waf(self):
    schemes = [
        self.matchHeader(('X-Engine', 'onMessage Shield')),
        self.matchContent(r'Blackbaud K\-12 conducts routine maintenance'),
        self.matchContent(r'onMessage SHEILD'),
        self.matchContent(r'maintenance\.blackbaud\.com'),
        self.matchContent(r'status\.blackbaud\.com')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'CacheWall (Varnish)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', 'Varnish')),
        self.matchHeader(('X-Varnish', '.+')),
        self.matchHeader(('X-Cachewall-Action', '.+?')),
        self.matchHeader(('X-Cachewall-Reason', '.+?')),
        self.matchContent(r'security by cachewall'),
        self.matchContent(r'403 naughty.{0,10}?not nice!'),
        self.matchContent(r'varnish cache server')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Viettel (Cloudrity)'


def is_waf(self):
    schemes = [
        self.matchContent(r"Access Denied.{0,10}?Viettel WAF"),
        self.matchContent(r"cloudrity\.com\.(vn)?/"),
        self.matchContent(r"Viettel WAF System")
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Squarespace (Squarespace)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', 'Squarespace')),
        self.matchCookie(r'^SS_ANALYTICS_ID='),
        self.matchCookie(r'^SS_MATTR='),
        self.matchCookie(r'^SS_MID='),
        self.matchCookie(r'SS_CVT='),
        self.matchContent(r'status\.squarespace\.com'),
        self.matchContent(r'BRICK\-\d{2}')
    ]
    if any(i for i in schemes):
        return True 
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Instart DX (Instart Logic)'


def is_waf(self):
    schema1 = [
        self.matchHeader(('X-Instart-Request-ID', '.+')),
        self.matchHeader(('X-Instart-Cache', '.+')),
        self.matchHeader(('X-Instart-WL', '.+'))
    ]
    schema2 = [
        self.matchContent(r'the requested url was rejected'),
        self.matchContent(r'please consult with your administrator'),
        self.matchContent(r'your support id is')
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloudbric (Penta Security)'


def is_waf(self):
    schemes = [
        self.matchContent(r'<title>Cloudbric.{0,5}?ERROR!'),
        self.matchContent(r'Your request was blocked by Cloudbric'),
        self.matchContent(r'please contact Cloudbric Support'),
        self.matchContent(r'cloudbric\.zendesk\.com'),
        self.matchContent(r'Cloudbric Help Center'),
        self.matchContent(r'malformed request syntax.{0,4}?invalid request message framing.{0,4}?or deceptive request routing')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'CrawlProtect (Jean-Denis Brun)'


def is_waf(self):
    schemes = [
        self.matchCookie(r'^crawlprotecttag='),
        self.matchContent(r'<title>crawlprotect'),
        self.matchContent(r'this site is protected by crawlprotect')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Sucuri CloudProxy (Sucuri Inc.)'


def is_waf(self):
    schemes = [
        self.matchHeader(('X-Sucuri-ID', r'.+?')),
        self.matchHeader(('X-Sucuri-Cache', r'.+?')),
        self.matchHeader(('Server', r'Sucuri(\-Cloudproxy)?')),
        self.matchHeader(('X-Sucuri-Block', r'.+?'), attack=True),
        self.matchContent(r"Access Denied.{0,6}?Sucuri Website Firewall"),
        self.matchContent(r"<title>Sucuri WebSite Firewall.{0,6}?(CloudProxy)?.{0,6}?Access Denied"),
        self.matchContent(r"sucuri\.net/privacy\-policy"),
        self.matchContent(r"cdn\.sucuri\.net/sucuri[-_]firewall[-_]block\.css"),
        self.matchContent(r'cloudproxy@sucuri\.net')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'eEye SecureIIS (BeyondTrust)'


def is_waf(self):
    schemes = [
        self.matchContent(r'SecureIIS is an internet security application'),
        self.matchContent(r'Download SecureIIS Personal Edition'),
        self.matchContent(r'https?://www\.eeye\.com/Secure\-?IIS')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ModSecurity (SpiderLabs)'


def is_waf(self):
    schema1 = [
        self.matchHeader(('Server', r'(mod_security|Mod_Security|NOYB)')),
        self.matchContent(r'This error was generated by Mod.?Security'),
        self.matchContent(r'rules of the mod.security.module'),
        self.matchContent(r'mod.security.rules triggered'),
        self.matchContent(r'Protected by Mod.?Security'),
        self.matchContent(r'/modsecurity[\-_]errorpage/'),
        self.matchContent(r'modsecurity iis')
    ]
    schema2 = [
        self.matchReason('ModSecurity Action'),
        self.matchStatus(403)
    ]
    schema3 = [
        self.matchReason('ModSecurity Action'),
        self.matchStatus(406)
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    if all(i for i in schema3):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WP Cerber Security (Cerber Tech)'


def is_waf(self):
    schemes = [
        self.matchContent(r'your request looks suspicious or similar to automated'),
        self.matchContent(r'our server stopped processing your request'),
        self.matchContent(r'We.re sorry.{0,10}?you are not allowed to proceed'),
        self.matchContent(r'requests from spam posting software'),
        self.matchContent(r'<title>403 Access Forbidden')
        ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SecureSphere (Imperva Inc.)'


def is_waf(self):
    schemes = [
        self.matchContent(r'<(title|h2)>Error'),
        self.matchContent(r'The incident ID is'),
        self.matchContent(r"This page can't be displayed"),
        self.matchContent(r'Contact support for additional information')
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'FortiWeb (Fortinet)'


def is_waf(self):
    schema1 = [
        self.matchCookie(r'^FORTIWAFSID='),
        self.matchContent('.fgd_icon')
    ]
    schema2 = [
        self.matchContent('fgd_icon'),
        self.matchContent('web.page.blocked'),
        self.matchContent('url'),
        self.matchContent('attack.id'),
        self.matchContent('message.id'),
        self.matchContent('client.ip')
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NetScaler AppFirewall (Citrix Systems)'


def is_waf(self):
    schemes = [
        # This header can be obtained without attack mode
        self.matchHeader(('Via', r'NS\-CACHE')),
        # Cookies are set only when someone is authenticated.
        # Not much reliable since wafw00f isn't authenticating.
        self.matchCookie(r'^(ns_af=|citrix_ns_id|NSC_)'),
        self.matchContent(r'(NS Transaction|AppFW Session) id'),
        self.matchContent(r'Violation Category.{0,5}?APPFW_'),
        self.matchContent(r'Citrix\|NetScaler'),
        # Reliable but not all servers return this header
        self.matchHeader(('Cneonction', r'^(keep alive|close)'), attack=True),
        self.matchHeader(('nnCoection', r'^(keep alive|close)'), attack=True)
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Sabre Firewall (Sabre)'


def is_waf(self):
    schema1 = [
        self.matchContent(r'dxsupport\.sabre\.com')
    ]
    schema2 = [
        self.matchContent(r'<title>Application Firewall Error'),
        self.matchContent(r'add some important details to the email for us to investigate')
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Expression Engine (EllisLab)'


def is_waf(self):
    schemes = [
        # I have seen some sites use a tracking header and sets a cookie upon authentication
        # 'Set-Cookie: _exp_tracking=rufyhweiuitefgcxyniercyft5-6dctuxeygfr'
        self.matchCookie(r'^exp_track.+?='),
        # There are traces found where cookie is returning values like:
        # Set-Cookie: exp_last_query=834y8d73y94d8g983u4shn8u4shr3uh3
        # Set-Cookie: exp_last_id=b342b432b1a876r8
        self.matchCookie(r'^exp_last_.+?=', attack=True),
        # In-page fingerprints vary a lot in different sites. Hence these are not quite reliable.
        self.matchContent(r'invalid get data')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DenyALL (Rohde & Schwarz CyberSecurity)'


def is_waf(self):
    schemes = [
        self.matchStatus(200),
        self.matchReason('Condition Intercepted')
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

# NOTE: this priority list is used so that each check can be prioritized,
# so that the quick checks are done first and ones that require more
# requests, are done later


wafdetectionsprio = [
    'ACE XML Gateway (Cisco)',
    'aeSecure (aeSecure)',
    'AireeCDN (Airee)',
    'Airlock (Phion/Ergon)',
    'Alert Logic (Alert Logic)',
    'AliYunDun (Alibaba Cloud Computing)',
    'Anquanbao (Anquanbao)',
    'AnYu (AnYu Technologies)',
    'Approach (Approach)',
    'AppWall (Radware)',
    'Armor Defense (Armor)',
    'ArvanCloud (ArvanCloud)',
    'ASP.NET Generic (Microsoft)',
    'ASPA Firewall (ASPA Engineering Co.)',
    'Astra (Czar Securities)',
    'AWS Elastic Load Balancer (Amazon)',
    'AzionCDN (AzionCDN)',
    'Azure Front Door (Microsoft)',
    'Barikode (Ethic Ninja)',
    'Barracuda (Barracuda Networks)',
    'Bekchy (Faydata Technologies Inc.)',
    'Beluga CDN (Beluga)',
    'BIG-IP Local Traffic Manager (F5 Networks)',
    'BinarySec (BinarySec)',
    'BitNinja (BitNinja)',
    'BlockDoS (BlockDoS)',
    'Bluedon (Bluedon IST)',
    'BulletProof Security Pro (AITpro Security)',
    'CacheWall (Varnish)',
    'CacheFly CDN (CacheFly)',
    'Comodo cWatch (Comodo CyberSecurity)',
    'CdnNS Application Gateway (CdnNs/WdidcNet)',
    'ChinaCache Load Balancer (ChinaCache)',
    'Chuang Yu Shield (Yunaq)',
    'Cloudbric (Penta Security)',
    'Cloudflare (Cloudflare Inc.)',
    'Cloudfloor (Cloudfloor DNS)',
    'Cloudfront (Amazon)',
    'CrawlProtect (Jean-Denis Brun)',
    'DataPower (IBM)',
    'DenyALL (Rohde & Schwarz CyberSecurity)',
    'Distil (Distil Networks)',
    'DOSarrest (DOSarrest Internet Security)',
    'DotDefender (Applicure Technologies)',
    'DynamicWeb Injection Check (DynamicWeb)',
    'Edgecast (Verizon Digital Media)',
    'Eisoo Cloud Firewall (Eisoo)',
    'Expression Engine (EllisLab)',
    'BIG-IP AppSec Manager (F5 Networks)',
    'BIG-IP AP Manager (F5 Networks)',
    'Fastly (Fastly CDN)',
    'FirePass (F5 Networks)',
    'FortiWeb (Fortinet)',
    'GoDaddy Website Protection (GoDaddy)',
    'Greywizard (Grey Wizard)',
    'Huawei Cloud Firewall (Huawei)',
    'HyperGuard (Art of Defense)',
    'Imunify360 (CloudLinux)',
    'Incapsula (Imperva Inc.)',
    'IndusGuard (Indusface)',
    'Instart DX (Instart Logic)',
    'ISA Server (Microsoft)',
    'Janusec Application Gateway (Janusec)',
    'Jiasule (Jiasule)',
    'Kona SiteDefender (Akamai)',
    'KS-WAF (KnownSec)',
    'KeyCDN (KeyCDN)',
    'LimeLight CDN (LimeLight)',
    'LiteSpeed (LiteSpeed Technologies)',
    'Open-Resty Lua Nginx (FLOSS)',
    'Oracle Cloud (Oracle)',
    'Malcare (Inactiv)',
    'MaxCDN (MaxCDN)',
    'Mission Control Shield (Mission Control)',
    'ModSecurity (SpiderLabs)',
    'NAXSI (NBS Systems)',
    'Nemesida (PentestIt)',
    'NevisProxy (AdNovum)',
    'NetContinuum (Barracuda Networks)',
    'NetScaler AppFirewall (Citrix Systems)',
    'Newdefend (NewDefend)',
    'NexusGuard Firewall (NexusGuard)',
    'NinjaFirewall (NinTechNet)',
    'NullDDoS Protection (NullDDoS)',
    'NSFocus (NSFocus Global Inc.)',
    'OnMessage Shield (BlackBaud)',
    'Palo Alto Next Gen Firewall (Palo Alto Networks)',
    'PerimeterX (PerimeterX)',
    'PentaWAF (Global Network Services)',
    'pkSecurity IDS (pkSec)',
    'PT Application Firewall (Positive Technologies)',
    'PowerCDN (PowerCDN)',
    'Profense (ArmorLogic)',
    'Puhui (Puhui)',
    'Qiniu (Qiniu CDN)',
    'Reblaze (Reblaze)',
    'RSFirewall (RSJoomla!)',
    'RequestValidationMode (Microsoft)',
    'Sabre Firewall (Sabre)',
    'Safe3 Web Firewall (Safe3)',
    'Safedog (SafeDog)',
    'Safeline (Chaitin Tech.)',
    'SecKing (SecKing)',
    'eEye SecureIIS (BeyondTrust)',
    'SecuPress WP Security (SecuPress)',
    'SecureSphere (Imperva Inc.)',
    'Secure Entry (United Security Providers)',
    'SEnginx (Neusoft)',
    'ServerDefender VP (Port80 Software)',
    'Shield Security (One Dollar Plugin)',
    'Shadow Daemon (Zecure)',
    'SiteGround (SiteGround)',
    'SiteGuard (Sakura Inc.)',
    'Sitelock (TrueShield)',
    'SonicWall (Dell)',
    'UTM Web Protection (Sophos)',
    'Squarespace (Squarespace)',
    'SquidProxy IDS (SquidProxy)',
    'StackPath (StackPath)',
    'Sucuri CloudProxy (Sucuri Inc.)',
    'Tencent Cloud Firewall (Tencent Technologies)',
    'Teros (Citrix Systems)',
    'Trafficshield (F5 Networks)',
    'TransIP Web Firewall (TransIP)',
    'URLMaster SecurityCheck (iFinity/DotNetNuke)',
    'URLScan (Microsoft)',
    'UEWaf (UCloud)',
    'Varnish (OWASP)',
    'Viettel (Cloudrity)',
    'VirusDie (VirusDie LLC)',
    'Wallarm (Wallarm Inc.)',
    'WatchGuard (WatchGuard Technologies)',
    'WebARX (WebARX Security Solutions)',
    'WebKnight (AQTRONIX)',
    'WebLand (WebLand)',
    'RayWAF (WebRay Solutions)',
    'WebSEAL (IBM)',
    'WebTotem (WebTotem)',
    'West263 CDN (West263CDN)',
    'Wordfence (Defiant)',
    'WP Cerber Security (Cerber Tech)',
    'WTS-WAF (WTS)',
    '360WangZhanBao (360 Technologies)',
    'XLabs Security WAF (XLabs)',
    'Xuanwudun (Xuanwudun)',
    'Yundun (Yundun)',
    'Yunsuo (Yunsuo)',
    'Yunjiasu (Baidu Cloud Computing)',
    'YXLink (YxLink Technologies)',
    'Zenedge (Zenedge)',
    'ZScaler (Accenture)'
]



#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = '360WangZhanBao (360 Technologies)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'qianxin\-waf')),
        self.matchHeader(('WZWS-Ray', r'.+?')),
        self.matchHeader(('X-Powered-By-360WZB', r'.+?')),
        self.matchContent(r'wzws\-waf\-cgi/'),
        self.matchContent(r'wangshan\.360\.cn'),
        self.matchStatus(493)
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'BulletProof Security Pro (AITpro Security)'


def is_waf(self):
    schemes = [
        self.matchContent(r'\+?bpsMessage'),
        self.matchContent(r'403 Forbidden Error Page'),
        self.matchContent(r'If you arrived here due to a search')
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'AWS Elastic Load Balancer (Amazon)'


def is_waf(self):
    schemes = [
        self.matchHeader(('X-AMZ-ID', '.+?')),
        self.matchHeader(('X-AMZ-Request-ID', '.+?')),
        self.matchCookie(r'^aws.?alb='),
        self.matchHeader(('Server', r'aws.?elb'), attack=True)
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'BIG-IP AppSec Manager (F5 Networks)'


def is_waf(self):
    schemes = [
        self.matchContent('the requested url was rejected'),
        self.matchContent('please consult with your administrator')
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Open-Resty Lua Nginx (FLOSS)'


def is_waf(self):
    schema1 = [
        self.matchHeader(('Server', r'^openresty/[0-9\.]+?')),
        self.matchStatus(403)
    ]
    schema2 = [
        self.matchContent(r'openresty/[0-9\.]+?'),
        self.matchStatus(406)
    ]
    if all(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Safedog (SafeDog)'


def is_waf(self):
    schemes = [
        self.matchCookie(r'^safedog\-flow\-item='),
        self.matchHeader(('Server', 'Safedog')),
        self.matchContent(r'safedogsite/broswer_logo\.jpg'),
        self.matchContent(r'404\.safedog\.cn/sitedog_stat.html'),
        self.matchContent(r'404\.safedog\.cn/images/safedogsite/head\.png')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Approach (Approach)'


def is_waf(self):
    schemes = [
        # This method of detection is old (though most reliable), so we check it first
        self.matchContent(r'approach.{0,10}?web application (firewall|filtering)'),
        self.matchContent(r'approach.{0,10}?infrastructure team')
        ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Astra (Czar Securities)'


def is_waf(self):
    schemes = [
        self.matchCookie(r'^cz_astra_csrf_cookie'),
        self.matchContent(r'astrawebsecurity\.freshdesk\.com'),
        self.matchContent(r'www\.getastra\.com/assets/images')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Incapsula (Imperva Inc.)'


def is_waf(self):
    schemes = [
        self.matchCookie(r'^incap_ses.*?='),
        self.matchCookie(r'^visid_incap.*?='),
        self.matchContent(r'incapsula incident id'),
        self.matchContent(r'powered by incapsula'),
        self.matchContent(r'/_Incapsula_Resource')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NinjaFirewall (NinTechNet)'


def is_waf(self):
    schemes = [
        self.matchContent(r'<title>NinjaFirewall.{0,10}?\d{3}.forbidden'),
        self.matchContent(r'For security reasons?.{0,10}?it was blocked and logged')
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Wordfence (Defiant)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'wf[_\-]?WAF')),
        self.matchContent(r"Generated by Wordfence"),
        self.matchContent(r'broke one of (the )?Wordfence (advanced )?blocking rules'),
        self.matchContent(r"/plugins/wordfence")
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'BIG-IP AP Manager (F5 Networks)'


def is_waf(self):
    schema1 = [
        self.matchCookie('^LastMRH_Session'),
        self.matchCookie('^MRHSession')
    ]
    schema2 = [
        self.matchCookie('^MRHSession'),
        self.matchHeader(('Server', r'Big([-_])?IP'), attack=True)
    ]
    schema3 = [
        self.matchCookie('^F5_fullWT'),
        self.matchCookie('^F5_fullWT'),
        self.matchCookie('^F5_HT_shrinked')
    ]
    if all(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    if any(i for i in schema3):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloudflare (Cloudflare Inc.)'


def is_waf(self):
    schemes = [
        self.matchHeader(('server', 'cloudflare')),
        self.matchHeader(('server', r'cloudflare[-_]nginx')),
        self.matchHeader(('cf-ray', r'.+?')),
        self.matchCookie('__cfduid')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloudfront (Amazon)'


def is_waf(self):
    schemes = [
        # This is standard detection schema, checking the server header
        self.matchHeader(('Server', 'Cloudfront')),
        # Found samples returning 'Via: 1.1 58bfg7h6fg76h8fg7jhdf2.cloudfront.net (CloudFront)'
        self.matchHeader(('Via', r'([0-9\.]+?)? \w+?\.cloudfront\.net \(Cloudfront\)')),
        # The request token is sent along with this header, eg:
        # X-Amz-Cf-Id: sX5QSkbAzSwd-xx3RbJmxYHL3iVNNyXa1UIebDNCshQbHxCjVcWDww==
        self.matchHeader(('X-Amz-Cf-Id', '.+?'), attack=True),
        # This is another reliable fingerprint found on headers
        self.matchHeader(('X-Cache', 'Error from Cloudfront'), attack=True),
        # These fingerprints are found on the blockpage itself
        self.matchContent(r'Generated by cloudfront \(CloudFront\)')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'VirusDie (VirusDie LLC)'


def is_waf(self):
    schemes = [
        self.matchContent(r"cdn\.virusdie\.ru/splash/firewallstop\.png"),
        self.matchContent(r'copy.{0,10}?Virusdie\.ru')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'URLMaster SecurityCheck (iFinity/DotNetNuke)'


def is_waf(self):
    schema1 = [
        self.matchHeader(('X-UrlMaster-Debug', '.+')),
        self.matchHeader(('X-UrlMaster-Ex', '.+')),
    ]
    schema2 = [
        self.matchContent(r"Ur[li]RewriteModule"),
        self.matchContent(r'SecurityCheck')
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'PowerCDN (PowerCDN)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Via', r'(.*)?powercdn.com(.*)?')),
        self.matchHeader(('X-Cache', r'(.*)?powercdn.com(.*)?')),
        self.matchHeader(('X-CDN', r'PowerCDN'))
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'UTM Web Protection (Sophos)'


def is_waf(self):
    schema1 = [
        self.matchContent(r'www\.sophos\.com'),
        self.matchContent(r'Powered by.?(Sophos)? UTM Web Protection')
    ]
    schema2 = [
        self.matchContent(r'<title>Access to the requested URL was blocked'),
        self.matchContent(r'Access to the requested URL was blocked'),
        self.matchContent(r'incident was logged with the following log identifier'),
        self.matchContent(r'Inbound Anomaly Score exceeded'),
        self.matchContent(r'Your cache administrator is')
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Nemesida (PentestIt)'


def is_waf(self):
    schemes = [
        self.matchContent(r'@?nemesida(\-security)?\.com'),
        self.matchContent(r'Suspicious activity detected.{0,10}?Access to the site is blocked'),
        self.matchContent(r'nwaf@'),
        self.matchStatus(222)
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Alert Logic (Alert Logic)'


def is_waf(self):
    schemes = [
        self.matchContent(r'<(title|h\d{1})>requested url cannot be found'),
        self.matchContent(r'we are sorry.{0,10}?but the page you are looking for cannot be found'),
        self.matchContent(r'back to previous page'),
        self.matchContent(r'proceed to homepage'),
        self.matchContent(r'reference id'),
        ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''
# For keeping python2 support for now
from __future__ import print_function

import csv
import io
import json
import logging
import os
import random
import re
import sys
from collections import defaultdict
from optparse import OptionParser
from wafw00f.lib.asciiarts import *
from wafw00f import __version__, __license__
from wafw00f.manager import load_plugins
from wafw00f.wafprio import wafdetectionsprio
from wafw00f.lib.evillib import urlParser, waftoolsengine, def_headers

class WAFW00F(waftoolsengine):

    xsstring = '<script>alert("XSS");</script>'
    sqlistring = "UNION SELECT ALL FROM information_schema AND ' or SLEEP(5) or '"
    lfistring = '../../../../etc/passwd'
    rcestring = '/bin/cat /etc/passwd; ping 127.0.0.1; curl google.com'
    xxestring = '<!ENTITY xxe SYSTEM "file:///etc/shadow">]><pwn>&hack;</pwn>'

    def __init__(self, target='www.example.com', debuglevel=0, path='/',
                 followredirect=True, extraheaders={}, proxies=None):

        self.log = logging.getLogger('wafw00f')
        self.attackres = None
        waftoolsengine.__init__(self, target, debuglevel, path, proxies, followredirect, extraheaders)
        self.knowledge = dict(generic=dict(found=False, reason=''), wafname=list())

    def normalRequest(self):
        return self.Request()

    def customRequest(self, headers=None):
        return self.Request(headers=headers)

    def nonExistent(self):
        return self.Request(path=self.path + str(random.randrange(100, 999)) + '.html')

    def xssAttack(self):
        return self.Request(path=self.path, params= {'s': self.xsstring})

    def xxeAttack(self):
        return self.Request(path=self.path, params= {'s': self.xxestring})

    def lfiAttack(self):
        return self.Request(path=self.path + self.lfistring)

    def centralAttack(self):
        return self.Request(path=self.path, params={'a': self.xsstring, 'b': self.sqlistring, 'c': self.lfistring})

    def sqliAttack(self):
        return self.Request(path=self.path, params= {'s': self.sqlistring})

    def oscAttack(self):
        return self.Request(path=self.path, params= {'s': self.rcestring})

    def performCheck(self, request_method):
        r = request_method()
        if r is None:
            raise RequestBlocked()
        return r

    # Most common attacks used to detect WAFs
    attcom = [xssAttack, sqliAttack, lfiAttack]
    attacks = [xssAttack, xxeAttack, lfiAttack, sqliAttack, oscAttack]

    def genericdetect(self):
        reason = ''
        reasons = ['Blocking is being done at connection/packet level.',
                   'The server header is different when an attack is detected.',
                   'The server returns a different response code when an attack string is used.',
                   'It closed the connection for a normal request.',
                   'The response was different when the request wasn\'t made from a browser.'
                ]
        try:
            # Testing for no user-agent response. Detects almost all WAFs out there.
            resp1 = self.performCheck(self.normalRequest)
            if 'User-Agent' in self.headers:
                del self.headers['User-Agent']  # Deleting the user-agent key from object not dict.
            resp3 = self.customRequest(headers=def_headers)
            if resp1.status_code != resp3.status_code:
                self.log.info('Server returned a different response when request didn\'t contain the User-Agent header.')
                reason = reasons[4]
                reason += '\r\n'
                reason += 'Normal response code is "%s",' % resp1.status_code
                reason += ' while the response code to a modified request is "%s"' % resp3.status_code
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return True

            # Testing the status code upon sending a xss attack
            resp2 = self.performCheck(self.xssAttack)
            if resp1.status_code != resp2.status_code:
                self.log.info('Server returned a different response when a XSS attack vector was tried.')
                reason = reasons[2]
                reason += '\r\n'
                reason += 'Normal response code is "%s",' % resp1.status_code
                reason += ' while the response code to cross-site scripting attack is "%s"' % resp2.status_code
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return True

            # Testing the status code upon sending a lfi attack
            resp2 = self.performCheck(self.lfiAttack)
            if resp1.status_code != resp2.status_code:
                self.log.info('Server returned a different response when a directory traversal was attempted.')
                reason = reasons[2]
                reason += '\r\n'
                reason += 'Normal response code is "%s",' % resp1.status_code
                reason += ' while the response code to a file inclusion attack is "%s"' % resp2.status_code
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return True

            # Testing the status code upon sending a sqli attack
            resp2 = self.performCheck(self.sqliAttack)
            if resp1.status_code != resp2.status_code:
                self.log.info('Server returned a different response when a SQLi was attempted.')
                reason = reasons[2]
                reason += '\r\n'
                reason += 'Normal response code is "%s",' % resp1.status_code
                reason += ' while the response code to a SQL injection attack is "%s"' % resp2.status_code
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return True

            # Checking for the Server header after sending malicious requests
            response = self.attackres
            normalserver = resp1.headers.get('Server')
            attackresponse_server = response.headers.get('Server')
            if attackresponse_server:
                if attackresponse_server != normalserver:
                    self.log.info('Server header changed, WAF possibly detected')
                    self.log.debug('Attack response: %s' % attackresponse_server)
                    self.log.debug('Normal response: %s' % normalserver)
                    reason = reasons[1]
                    reason += '\r\nThe server header for a normal response is "%s",' % normalserver
                    reason += ' while the server header a response to an attack is "%s",' % attackresponse_server
                    self.knowledge['generic']['reason'] = reason
                    self.knowledge['generic']['found'] = True
                    return True

        # If at all request doesn't go, press F
        except RequestBlocked:
            self.knowledge['generic']['reason'] = reasons[0]
            self.knowledge['generic']['found'] = True
            return True
        return False

    def matchHeader(self, headermatch, attack=False):
        if attack:
            r = self.attackres
        else: r = rq
        if r is None:
            return
        header, match = headermatch
        headerval = r.headers.get(header)
        if headerval:
            # set-cookie can have multiple headers, python gives it to us
            # concatinated with a comma
            if header == 'Set-Cookie':
                headervals = headerval.split(', ')
            else:
                headervals = [headerval]
            for headerval in headervals:
                if re.search(match, headerval, re.I):
                    return True
        return False

    def matchStatus(self, statuscode, attack=True):
        if attack:
            r = self.attackres
        else: r = rq
        if r is None:
            return
        if r.status_code == statuscode:
            return True
        return False

    def matchCookie(self, match, attack=False):
        return self.matchHeader(('Set-Cookie', match), attack=attack)

    def matchReason(self, reasoncode, attack=True):
        if attack:
            r = self.attackres
        else: r = rq
        if r is None:
            return
        # We may need to match multiline context in response body
        if str(r.reason) == reasoncode:
            return True
        return False

    def matchContent(self, regex, attack=True):
        if attack:
            r = self.attackres
        else: r = rq
        if r is None:
            return
        # We may need to match multiline context in response body
        if re.search(regex, r.text, re.I):
            return True
        return False

    wafdetections = dict()

    plugin_dict = load_plugins()
    result_dict = {}
    for plugin_module in plugin_dict.values():
        wafdetections[plugin_module.NAME] = plugin_module.is_waf
    # Check for prioritized ones first, then check those added externally
    checklist = wafdetectionsprio
    checklist += list(set(wafdetections.keys()) - set(checklist))

    def identwaf(self, findall=False):
        detected = list()
        try:
            self.attackres = self.performCheck(self.centralAttack)
        except RequestBlocked:
            return detected
        for wafvendor in self.checklist:
            self.log.info('Checking for %s' % wafvendor)
            if self.wafdetections[wafvendor](self):
                detected.append(wafvendor)
                if not findall:
                    break
        self.knowledge['wafname'] = detected
        return detected

def calclogginglevel(verbosity):
    default = 40  # errors are printed out
    level = default - (verbosity * 10)
    if level < 0:
        level = 0
    return level

def buildResultRecord(url, waf):
    result = {}
    result['url'] = url
    if waf:
        result['detected'] = True
        if waf == 'generic':
            result['firewall'] = 'Generic'
            result['manufacturer'] = 'Unknown'
        else:
            result['firewall'] = waf.split('(')[0].strip()
            result['manufacturer'] = waf.split('(')[1].replace(')', '').strip()
    else:
        result['detected'] = False
        result['firewall'] = 'None'
        result['manufacturer'] = 'None'
    return result

def getTextResults(res=None):
    # leaving out some space for future possibilities of newer columns
    # newer columns can be added to this tuple below
    keys = ('detected')
    res = [({key: ba[key] for key in ba if key not in keys}) for ba in res]
    rows = []
    for dk in res:
        p = [str(x) for _, x in dk.items()]
        rows.append(p)
    for m in rows:
        m[1] = '%s (%s)' % (m[1], m[2])
        m.pop()
    defgen = [
        (max([len(str(row[i])) for row in rows]) + 3)
        for i in range(len(rows[0]))
    ]
    rwfmt = "".join(["{:>"+str(dank)+"}" for dank in defgen])
    textresults = []
    for row in rows:
        textresults.append(rwfmt.format(*row))
    return textresults

def disableStdOut():
    sys.stdout = None

def enableStdOut():
    sys.stdout = sys.__stdout__

def getheaders(fn):
    headers = {}
    if not os.path.exists(fn):
        logging.getLogger('wafw00f').critical('Headers file "%s" does not exist!' % fn)
        return
    with io.open(fn, 'r', encoding='utf-8') as f:
        for line in f.readlines():
            _t = line.split(':', 2)
            if len(_t) == 2:
                h, v = map(lambda x: x.strip(), _t)
                headers[h] = v
    return headers

class RequestBlocked(Exception):
    pass

def main():
    parser = OptionParser(usage='%prog url1 [url2 [url3 ... ]]\r\nexample: %prog http://www.victim.org/')
    parser.add_option('-v', '--verbose', action='count', dest='verbose', default=0,
                      help='Enable verbosity, multiple -v options increase verbosity')
    parser.add_option('-a', '--findall', action='store_true', dest='findall', default=False,
                      help='Find all WAFs which match the signatures, do not stop testing on the first one')
    parser.add_option('-r', '--noredirect', action='store_false', dest='followredirect',
                      default=True, help='Do not follow redirections given by 3xx responses')
    parser.add_option('-t', '--test', dest='test', help='Test for one specific WAF')
    parser.add_option('-o', '--output', dest='output', help='Write output to csv, json or text file depending on file extension. For stdout, specify - as filename.',
                      default=None)
    parser.add_option('-i', '--input-file', dest='input', help='Read targets from a file. Input format can be csv, json or text. For csv and json, a `url` column name or element is required.',
                      default=None)
    parser.add_option('-l', '--list', dest='list', action='store_true',
                      default=False, help='List all WAFs that WAFW00F is able to detect')
    parser.add_option('-p', '--proxy', dest='proxy', default=None,
                      help='Use an HTTP proxy to perform requests, examples: http://hostname:8080, socks5://hostname:1080, http://user:pass@hostname:8080')
    parser.add_option('--version', '-V', dest='version', action='store_true',
                      default=False, help='Print out the current version of WafW00f and exit.')
    parser.add_option('--headers', '-H', dest='headers', action='store', default=None,
                      help='Pass custom headers via a text file to overwrite the default header set.')
    options, args = parser.parse_args()
    logging.basicConfig(level=calclogginglevel(options.verbose))
    log = logging.getLogger('wafw00f')
    if options.output == '-':
        disableStdOut()
    print(randomArt())
    if options.list:
        print('[+] Can test for these WAFs:\r\n')
        attacker = WAFW00F(None)
        try:
            m = [i.replace(')', '').split(' (') for i in wafdetectionsprio]
            print(R+'  WAF Name'+' '*24+'Manufacturer\n  '+'-'*8+' '*24+'-'*12+'\n')
            max_len = max(len(str(x)) for k in m for x in k) 
            for inner in m:
                first = True
                for elem in inner:
                    if first:
                        text = Y+"  {:<{}} ".format(elem, max_len+2)
                        first = False
                    else:
                        text = W+"{:<{}} ".format(elem, max_len+2)
                    print(text, E, end="")
                print()
            sys.exit(0)
        except Exception:
            return
    if options.version:
        print('[+] The version of WAFW00F you have is %sv%s%s' % (B, __version__, E))
        print('[+] WAFW00F is provided under the %s%s%s license.' % (C, __license__, E))
        return
    extraheaders = {}
    if options.headers:
        log.info('Getting extra headers from %s' % options.headers)
        extraheaders = getheaders(options.headers)
        if extraheaders is None:
            parser.error('Please provide a headers file with colon delimited header names and values')
    if len(args) == 0 and not options.input:
        parser.error('No test target specified.')
    #check if input file is present
    if options.input:
        log.debug("Loading file '%s'" % options.input)
        try:
            if options.input.endswith('.json'):
                with open(options.input) as f:
                    try:
                        urls = json.loads(f.read())
                    except json.decoder.JSONDecodeError:
                        log.critical("JSON file %s did not contain well-formed JSON", options.input)
                        sys.exit(1)
                log.info("Found: %s urls to check." %(len(urls)))
                targets = [ item['url'] for item in urls ]
            elif options.input.endswith('.csv'):
                columns = defaultdict(list)
                with open(options.input) as f:
                    reader = csv.DictReader(f) 
                    for row in reader:
                        for (k,v) in row.items():
                            columns[k].append(v)
                targets = columns['url']
            else:
                with open(options.input) as f:
                    targets = [x for x in f.read().splitlines()]
        except FileNotFoundError:
            log.error('File %s could not be read. No targets loaded.', options.input)
            sys.exit(1)
    else:
        targets = args
    results = []
    for target in targets:
        if not target.startswith('http'):
            log.info('The url %s should start with http:// or https:// .. fixing (might make this unusable)' % target)
            target = 'https://' + target
        print('[*] Checking %s' % target)
        pret = urlParser(target)
        if pret is None:
            log.critical('The url %s is not well formed' % target)
            sys.exit(1)
        (hostname, _, path, _, _) = pret
        log.info('starting wafw00f on %s' % target)
        proxies = dict()
        if options.proxy:
            proxies = {
                "http": options.proxy,
                "https": options.proxy,
            }
        attacker = WAFW00F(target, debuglevel=options.verbose, path=path,
                    followredirect=options.followredirect, extraheaders=extraheaders,
                        proxies=proxies)
        global rq
        rq = attacker.normalRequest()
        if rq is None:
            log.error('Site %s appears to be down' % hostname)
            continue
        if options.test:
            if options.test in attacker.wafdetections:
                waf = attacker.wafdetections[options.test](attacker)
                if waf:
                    print('[+] The site %s%s%s is behind %s%s%s WAF.' % (B, target, E, C, options.test, E))
                else:
                    print('[-] WAF %s was not detected on %s' % (options.test, target))
            else:
                print('[-] WAF %s was not found in our list\r\nUse the --list option to see what is available' % options.test)
            return
        waf = attacker.identwaf(options.findall)
        log.info('Identified WAF: %s' % waf)
        if len(waf) > 0:
            for i in waf:
                results.append(buildResultRecord(target, i))
            print('[+] The site %s%s%s is behind %s%s%s WAF.' % (B, target, E, C, (E+' and/or '+C).join(waf), E))
        if (options.findall) or len(waf) == 0:
            print('[+] Generic Detection results:')
            if attacker.genericdetect():
                log.info('Generic Detection: %s' % attacker.knowledge['generic']['reason'])
                print('[*] The site %s seems to be behind a WAF or some sort of security solution' % target)
                print('[~] Reason: %s' % attacker.knowledge['generic']['reason'])
                results.append(buildResultRecord(target, 'generic'))
            else:
                print('[-] No WAF detected by the generic detection')
                results.append(buildResultRecord(target, None))
        print('[~] Number of requests: %s' % attacker.requestnumber)
    #print table of results
    if len(results) > 0:
        log.info("Found: %s matches." % (len(results)))
    if options.output:
        if options.output == '-':
            enableStdOut()
            print(os.linesep.join(getTextResults(results)))
        elif options.output.endswith('.json'):
            log.debug("Exporting data in json format to file: %s" % (options.output))
            with open(options.output, 'w') as outfile:
                json.dump(results, outfile, indent=2)
        elif options.output.endswith('.csv'):
            log.debug("Exporting data in csv format to file: %s" % (options.output))
            with open(options.output, 'w') as outfile:
                csvwriter = csv.writer(outfile, delimiter=',', quotechar='"', 
                    quoting=csv.QUOTE_MINIMAL)
                count = 0
                for result in results:
                    if count == 0:
                        header = result.keys()
                        csvwriter.writerow(header)
                        count += 1
                    csvwriter.writerow(result.values())
        else:
            log.debug("Exporting data in text format to file: %s" % (options.output))
            with open(options.output, 'w') as outfile:
                outfile.write(os.linesep.join(getTextResults(results)))

if __name__ == '__main__':
    if sys.hexversion < 0x2060000:
        sys.stderr.write('Your version of python is way too old... please update to 2.6 or later\r\n')
    main()



#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ASPA Firewall (ASPA Engineering Co.)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'ASPA[\-_]?WAF')),
        self.matchHeader(('ASPA-Cache-Status', r'.+?'))
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Eisoo Cloud Firewall (Eisoo)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'EisooWAF(\-AZURE)?/?')),
        self.matchContent(r'<link.{0,10}?href=\"/eisoo\-firewall\-block\.css'),
        self.matchContent(r'www\.eisoo\.com'),
        self.matchContent(r'&copy; \d{4} Eisoo Inc')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Malcare (Inactiv)'


def is_waf(self):
    schemes = [
        self.matchContent(r'firewall.{0,15}?powered.by.{0,15}?malcare.{0,15}?pro'),
        self.matchContent('blocked because of malicious activities')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NAXSI (NBS Systems)'


def is_waf(self):
    schemes = [
        self.matchHeader(('X-Data-Origin', r'^naxsi(.+)?')),
        self.matchHeader(('Server', r'naxsi(.+)?')),
        self.matchContent(r'blocked by naxsi'),
        self.matchContent(r'naxsi blocked information')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ASP.NET Generic (Microsoft)'


def is_waf(self):
    schemes = [
        self.matchContent(r'iis (\d+.)+?detailed error'),
        self.matchContent(r'potentially dangerous request querystring'),
        self.matchContent(r'application error from being viewed remotely (for security reasons)?'),
        self.matchContent(r'An application error occurred on the server'),
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SquidProxy IDS (SquidProxy)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'squid(/[0-9\.]+)?')),
        self.matchContent(r'Access control configuration prevents your request')
        ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Sitelock (TrueShield)'

# Well this is confusing, Sitelock itself uses Incapsula from Imperva
# So the fingerprints obtained on blockpage are similar to those of Incapsula.

def is_waf(self):
    schemes = [
        self.matchContent(r"SiteLock will remember you"),
        self.matchContent(r"Sitelock is leader in Business Website Security Services"),
        self.matchContent(r"sitelock[_\-]shield([_\-]logo|[\-_]badge)?"),
        self.matchContent(r'SiteLock incident ID')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Shadow Daemon (Zecure)'


def is_waf(self):
    schemes = [
        self.matchContent(r"<h\d{1}>\d{3}.forbidden<.h\d{1}>"),
        self.matchContent(r"request forbidden by administrative rules")
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Jiasule (Jiasule)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'jiasule\-waf')),
        self.matchCookie(r'^jsl_tracking(.+)?='),
        self.matchCookie(r'__jsluid='),
        self.matchContent(r'notice\-jiasule'),
        self.matchContent(r'static\.jiasule\.com')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Huawei Cloud Firewall (Huawei)'


def is_waf(self):
    schemes = [
        self.matchCookie(r'^HWWAFSESID='),
        self.matchHeader(('Server', r'HuaweiCloudWAF')),
        self.matchContent(r'hwclouds\.com'),
        self.matchContent(r'hws_security@')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Imunify360 (CloudLinux)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'imunify360.{0,10}?')),
        self.matchContent(r'protected.by.{0,10}?imunify360'),
        self.matchContent(r'powered.by.{0,10}?imunify360'),
        self.matchContent(r'imunify360.preloader')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Reblaze (Reblaze)'


def is_waf(self):
    schema1 = [
        self.matchCookie(r'^rbzid'),
        self.matchHeader(('Server', 'Reblaze Secure Web Gateway'))
    ]
    schema2 = [
        self.matchContent(r'current session has been terminated'),
        self.matchContent(r'do not hesitate to contact us'),
        self.matchContent(r'access denied \(\d{3}\)')
    ]
    if any(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'PT Application Firewall (Positive Technologies)'


def is_waf(self):
    schemes = [
        self.matchContent(r'<h1.{0,10}?Forbidden'),
        self.matchContent(r'<pre>Request.ID:.{0,10}?\d{4}\-(\d{2})+.{0,15}?pre>')
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Yundun (Yundun)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', 'YUNDUN')),
        self.matchHeader(('X-Cache', 'YUNDUN')),
        self.matchCookie(r'^yd_cookie='),
        self.matchContent(r'Blocked by YUNDUN Cloud WAF'),
        self.matchContent(r'yundun\.com/yd[-_]http[_-]error/'),
        self.matchContent(r'www\.yundun\.com/(static/js/fingerprint\d{1}?\.js)?')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'StackPath (StackPath)'


def is_waf(self):
    schemes = [
        self.matchContent(r"This website is using a security service to protect itself"),
        self.matchContent(r'You performed an action that triggered the service and blocked your request')
    ]
    if all(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Comodo cWatch (Comodo CyberSecurity)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'Protected by COMODO WAF(.+)?'))
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

import io
from setuptools import setup, find_packages
from os import path
this_directory = path.abspath(path.dirname(__file__))
with io.open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    desc = f.read()

setup(
    name='wafw00f',
    version=__import__('wafw00f').__version__,
    description='The Web Application Firewall Detection and Fingerprinting Toolkit.',
    long_description=desc,
    long_description_content_type='text/markdown',
    author='Sandro Gauci',
    author_email='sandro@enablesecurity.com',
    license='BSD License',
    url='https://github.com/enablesecurity/wafw00f',
    project_urls={
        "Bug Tracker": "https://github.com/EnableSecurity/wafw00f/issues",
        "Documentation": "https://github.com/EnableSecurity/wafw00f/wiki",
        "Source Code": "https://github.com/EnableSecurity/wafw00f/tree/master",
    },
    packages=find_packages(),
    scripts=['wafw00f/bin/wafw00f'],
    install_requires=[
        'requests',
        'requests[socks]',
        'pluginbase'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Topic :: Internet',
        'Topic :: Security',
        'Topic :: System :: Networking :: Firewalls',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent'
    ],
    keywords='waf firewall detector fingerprint',
    extras_require={
        'dev': [
            'prospector',
        ],
        'docs': [
            'Sphinx',
        ],
    },
)



#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NexusGuard Firewall (NexusGuard)'


def is_waf(self):
    schemes = [
        self.matchContent(r'Powered by Nexusguard'),
        self.matchContent(r'nexusguard\.com/wafpage/.+#\d{3};')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SonicWall (Dell)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', 'SonicWALL')),
        self.matchContent(r"<(title|h\d{1})>Web Site Blocked"),
        self.matchContent(r'\+?nsa_banner')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Newdefend (NewDefend)'


def is_waf(self):
    schemes = [
        # This header can be obtained without attack mode
        # Most reliable fingerprint
        self.matchHeader(('Server', 'Newdefend')),
        # Reliable ones within blockpage
        self.matchContent(r'www\.newdefend\.com/feedback'),
        self.matchContent(r'/nd\-block/')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Greywizard (Grey Wizard)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', 'greywizard')),
        self.matchContent(r'<(title|h\d{1})>Grey Wizard'),
        self.matchContent(r'contact the website owner or Grey Wizard'),
        self.matchContent(r'We.ve detected attempted attack or non standard traffic from your ip address')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers 
See the LICENSE file for copying permission.
'''

NAME = 'Palo Alto Next Gen Firewall (Palo Alto Networks)'


def is_waf(self):
    schemes = [
        self.matchContent(r'Download of virus.spyware blocked'),
        self.matchContent(r'Palo Alto Next Generation Security Platform')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Barracuda (Barracuda Networks)'


def is_waf(self):
    schemes = [
        self.matchCookie(r'^barra_counter_session='),
        self.matchCookie(r'^BNI__BARRACUDA_LB_COOKIE='),
        self.matchCookie(r'^BNI_persistence='),
        self.matchCookie(r'^BN[IE]S_.*?='),
        self.matchContent(r'Barracuda.Networks')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Bluedon (Bluedon IST)'


def is_waf(self):
    schemes = [
        # Found sample servers returning 'Server: BDWAF/2.0'
        self.matchHeader(('Server', r'BDWAF')),
        self.matchContent(r'bluedon web application firewall')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DynamicWeb Injection Check (DynamicWeb)'


def is_waf(self):
    schemes = [
        self.matchHeader(('X-403-Status-By', r'dw.inj.check'), attack=True),
        self.matchContent(r'by dynamic check(.{0,10}?module)?')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'UEWaf (UCloud)'


def is_waf(self):
    schemes = [
        self.matchHeader(('Server', r'uewaf(/[0-9\.]+)?')),
        self.matchContent(r'/uewaf_deny_pages/default/img/'),
        self.matchContent(r'ucloud\.cn')
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'LiteSpeed (LiteSpeed Technologies)'


def is_waf(self):
    schema1 = [
        self.matchHeader(('Server', 'LiteSpeed')),
        self.matchStatus(403)
    ]
    schema2 = [
        self.matchContent(r'Proudly powered by litespeed web server'),
        self.matchContent(r'www\.litespeedtech\.com/error\-page')
    ]
    if all(i for i in schema1):
        return True
    if any(i for i in schema2):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WebKnight (AQTRONIX)'


def is_waf(self):
    schema1 = [
        self.matchStatus(999),
        self.matchReason('No Hacking')
    ]    
    schema2 = [
        self.matchStatus(404),
        self.matchReason('Hack Not Found')
    ]
    schema3 = [
        self.matchContent(r'WebKnight Application Firewall Alert'),
        self.matchContent(r'What is webknight\?'),
        self.matchContent(r'AQTRONIX WebKnight is an application firewall'),
        self.matchContent(r'WebKnight will take over and protect'),
        self.matchContent(r'aqtronix\.com/WebKnight'),
        self.matchContent(r'AQTRONIX.{0,10}?WebKnight'),
    ]
    if all(i for i in schema1):
        return True
    if all(i for i in schema2):
        return True    
    if any(i for i in schema3):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Shield Security (One Dollar Plugin)'


def is_waf(self):
    schemes = [
        self.matchContent(r"You were blocked by the Shield"),
        self.matchContent(r"remaining transgression\(s\) against this site"),
        self.matchContent(r"Something in the URL.{0,5}?Form or Cookie data wasn\'t appropriate")
    ]
    if any(i for i in schemes):
        return True
    return False


#!/usr/bin/env python
'''
Copyright (C) 2020, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'pkSecurity IDS (pkSec)'


def is_waf(self):
    schema1 = [
        self.matchContent(r'pk.?Security.?Module'),
        self.matchContent(r'Security.Alert')
    ]
    schema2 = [
        self.matchContent(r'As this could be a potential hack attack'),
        self.matchContent(r'A safety critical (call|request) was (detected|discovered) and blocked'),
        self.matchContent(r'maximum number of reloads per minute and prevented access')
    ]
    if any(i for i in schema2):
        return True
    if all(i for i in schema1):
        return True
    return False